/*****************************************************************************
*                     Copyright (c)  Gemplus Developpement 1992 - 1997
*
* Name        : TLPREADER.H
*
* Description : 
*
  Author      : JL-GIRAUD
  Compiler    : Visual C++, C Linux
  Host        : 

* $Id: tlpreader.h,v 1.5 2000/12/11 17:53:31 rousseau Exp $
* $Log: tlpreader.h,v $
* Revision 1.5  2000/12/11 17:53:31  rousseau
* Added a 5 seconds timeout if no reader present
*
* Revision 1.3  2000/09/04 15:31:26  giraud
* Change buffer size in tlpreader.* and rftlpreader.*
* Add lib/samples clean-up when make clean is entered in lib/
*
* Revision 1.2  2000/09/04 08:19:06  rousseau
* code cleanup, changed the API and name for [Init|Close]Serial
*
*
******************************************************************************
*
* Warning     : 
* Remarks     : 
*
*****************************************************************************/


#ifndef _TLP_READER_H
#define _TLP_READER_H

/****** Buffer Size ******/
#define TLP_ATR_BUF_SIZE 255
#define TLP_PW_DOWN_BUF_SIZE 100
#define TLP_DATA_BUF_SIZE 600
#define TLP_SW_BUF_SIZE 20


/****** TLP PART ******/

#define TLP_EOT      0x03
#define TLP_TRANS_OK 0x60

/* TLP ERRORS */ 
  
#define   TLP_NO_ERROR                          0x00
#define   TLP_UNKNOWN_COMMAND                   0x01
#define   TLP_DRIVER_LIMITATION                 0x02
#define   TLP_WRONG_ARG_LENGTH                  0x03
#define   TLP_COMMAND_UNKNOWN                   0x04
#define   TLP_PROTOCOL_ERROR                    0x09
#define   TLP_WRONG_ATR                         0x10
#define   TLP_WRONG_RD_INS                      0x11
#define   TLP_MSG_TOO_LONG                      0x12
#define   TLP_READ_ERROR                        0x13
#define   TLP_CARD_OFF                          0x15
#define   TLP_PROG_VOLT_ERROR                   0x16
#define   TLP_UNKNOWN_PROTOCOL1                 0x17
#define   TLP_UNKNOWN_PROTOCOL2                 0x18
#define   TLP_ILLEGAL_ACCESS                    0x19
#define   TLP_LENGTH_ERROR                      0x1A
#define   TLP_WRONG_NB_OF_PARAM                 0x1B
#define   TLP_TCK_ERROR                         0x1D
#define   TLP_PROTECTION_ERROR                  0x1E
#define   TLP_WRONG_DATA                        0x1F
#define   TLP_UNKNOWN_CARD_PROTOCOL             0xA0
#define   TLP_CARD_MUTE                         0xA2
#define   TLP_CARD_PARITY_ERROR                 0xA3
#define   TLP_CARD_WRONG_PROCEDURE_BYTE         0xE4
#define   TLP_TRANSFERT_TO_CARD_INTERRUPTED     0xE5
#define   TLP_CARD_RETURNED_ERROR               0xE7
#define   TLP_CARD_REMOVED                      0xF7
#define   TLP_SHORT_CIRCUIT                     0xF8
#define   TLP_CARD_ABSENT                       0xFB

/*****  TLP COMMANDS *****/

extern const char TLP_POWER_DOWN[9];
extern const char TLP_POWER_UP[15];

/* TLP offsets in messages */
#define TLP_ACK_OFF           0
#define TLP_LENGTH_OFF        2
#define TLP_CMD_OFF           4
#define TLP_DATA_OFF          6

/*#define TLP_ATR_LENGTH_OFF    10*/
#define TLP_ATR_OFF           12 /* in chars */
#define TLP_ATR_LENGTH_ADJUST 3  /* in bytes */

int TLPResetCardAscii(TYPE_HANDLE fd, char* atr, int length);

int TLPResetCardBinary(TYPE_HANDLE fd, unsigned char* atr, int length);

int TLPGetReaderResponse(TYPE_HANDLE fd, char *buffer, int length);

t_rd_error TLPTrError(int reader_error);

int TLPSendIncomingAPDUBinary(TYPE_HANDLE fd,
	unsigned char CLA,
	unsigned char INS,
	unsigned char P1,
	unsigned char P2,
	unsigned char L,
	const unsigned char *data_in,
	unsigned char *ad_SW1,
	unsigned char *ad_SW2);

int TLPSendOutgoingAPDUBinary(TYPE_HANDLE fd,
	unsigned char CLA,
	unsigned char INS,
	unsigned char P1,
	unsigned char P2,
	unsigned char L,
	unsigned char *data_out,
	unsigned char *ad_SW1,
	unsigned char *ad_SW2);

int TLPSendIncomingOutgoingAPDUBinary(TYPE_HANDLE fd,
	unsigned char CLA,
	unsigned char INS,
	unsigned char P1,
	unsigned char P2,
	unsigned char L_in,
	const unsigned char *data_in,
	unsigned char L_out,
	unsigned char *data_out,
	unsigned char *ad_SW1,
	unsigned char *ad_SW2);

int TLPSendIncomingAPDUAscii(TYPE_HANDLE fd, const char* APDU_Data,
	char* ad_response, int resp_length);

int TLPSendOutgoingAPDUAscii(TYPE_HANDLE fd,  const char* APDU_Data,  
	char* ad_response, int resp_length);

int TLPSendIncomingOutgoingAPDUAscii(TYPE_HANDLE fd, const char* APDU_Data,
	char* ad_response, int resp_length);

int TLPPowerDown(TYPE_HANDLE fd);

#endif

